{
Copyright (c) 1998-2001 Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
     must display the following acknowledgement:
     This product includes software developed by Jonathan Revusky
4. The name of the author may not be used to endorse or promote products
     derived from this software without specific prior written permission.
5. The Copyright Banner variable in the source code is not to be modified.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
}

unit Compile;

interface

    procedure RunJCImpl;

implementation

uses
  Windows, Classes, SysUtils, SmartJGlobals,
  Jutils, JavaRuntime;

type
  PPChar = ^PChar;
  TCMain = function (argc : Integer; argv : PPChar) : Integer; stdcall;

var
  isVerbose, forceRecompile, NoJikes : Boolean;
  outputDirectory : String; // set if the user inputs the -d option. includes trailing \
  totalFiles : Integer;
  JikesMain : TCMain;
  JikesDLL : THandle;
  foundAFile : Boolean;

  procedure Banner;
  begin
    writeln('SmartJC Java compilation utility, version 0.60');
    writeln('Copyright (C) Jonathan Revusky, 1998-2001');
    writeln('Send suggestions and feedback to jon@revusky.com');
    writeln('');
  end;

  procedure ShowUsage;
    var
      ProgName : String;
    begin
      ProgName := ExtractFileName(ParamStr(0));
      ProgName := Copy(ProgName, 1, length(ProgName) - length(ExtractFileExt(ProgName)));
      writeln('');
      writeln('  Usage: ' + ProgName + ' [-options] arguments');
      writeln('      Options*:');
      writeln('         -v:     Extra verbosity');
      writeln('         -f:     Force recompile (no file age check)');
      writeln('         -p:     Pause before exiting');
      writeln('         -1:     compile against java 1.1 if available (1.2 is default)');
      writeln('         -nojikes : Do not look for the jikes compiler. Use javac.');
      writeln('   *All regular javac options may be used.');
      writeln('   *Jikes-specific options such as +E etcetera are available when using jikes.');
      writeln('');
      writeln('The arguments may specify source files OR directories.');
      writeln('Indicating a directory implies a recursive recompile down the hierarchy.');
      writeln('By default, SmartJC will only compile source files that are newer');
      writeln('than their corresponding .class files.');
      writeln('');
      writeln('The most typical usage might be the line:  ' + ProgName + ' .');
      writeln('');
      write('Requires the Sun JDK 1.1 or higher or the Jikes engine for correct operation.');
    end;

  procedure ShowJikesMessage;
  begin
    writeln('Using the Jikes compilation engine (C) Copyright IBM Corp. 1997, 2001.');
    writeln('For more information about Jikes, please visit the following URL:');
    writeln('http://oss.software.ibm.com/developerworks/opensource/jikes/');
  end;

// takes as arguments a list of arguments and also jikes options.

  function createArgv(strings : TStrings) : PPChar;
  var
    PPC : PPChar;
    I : Integer;
  begin
    Result := allocMem((2 + Strings.Count) * sizeof(PChar));
    PPC := Result;
    PPC^ := PChar(ParamStr(0));
    for I := 0 to Strings.Count-1 do
    begin
      inc(PPC);
      PPC^ := PChar(Strings[I]);
    end;
    inc(PPC);
    PPC^ := Nil;
  end;

  procedure addFile(filename : String ;    Args : TStrings);
  begin
    if length(filename) = 0 then
      Exit;
    inc(TotalFiles);
    if forceRecompile or isOutOfDate(filename, outputDirectory) then
    begin
      Args.add(ExpandFilename(filename));
    end;
    FoundAFile := True;
  end;

  procedure AddDirectory(Dir : String ; Args : Tstrings);
  var
    SearchRec : TSearchRec;
    S : String;
  begin
    AppendBackSlash(Dir);
    if FindFirst(Dir + '*', faDirectory, searchRec) = 0 then
    repeat
      S := SearchRec.Name;
      if ((searchRec.Attr and faDirectory) <> 0) and (S[1] <> '.') then
        AddDirectory(Dir + S, Args)
      else
        if LowerCase(ExtractFileExt(S)) = '.java' then
          addFile(Dir + S, Args);
    until FindNext(SearchRec) <> 0;
    FindClose(SearchRec);
  end;

// Takes the command-line args that aren't options
// and expands them into a list of fully specified filenames.
// Args is the list of filenames to add to.

  procedure addParam(S : String; Args : TStrings);
  var
    SearchRec : TSearchRec;
    wildcards : Boolean;
    Fattr : Integer;
  begin
    wildcards := (Pos('*', S) >0) or (Pos('?', S) >0);
    if not wildcards then
    begin
        S := ExpandFileName(S);
        FAttr := FileGetAttr(S);
        if (FAttr <> -1) and ((Fattr and faDirectory) <> 0) then
            begin
                addDirectory(S, Args);
                Exit;
            end;
        AppendExtension(S, '.java');
        if FindFirst(S, 0, SearchRec) =0 then
            addFile(ExpandFilename(ExtractFilePath(S) + SearchRec.Name), Args);
        FindClose(SearchRec);
        Exit;
    end;

    AppendExtension(S, '.java');

    if FindFirst(S, 0, SearchRec) =0 then
    repeat
        addFile(ExpandFilename(ExtractFilePath(S) + SearchRec.Name), Args);
    until
        FindNext(SearchRec) <> 0
    else
      writeln('No file found matching ', S);
    FindClose(SearchRec);
  end;


// Big messy routine that processes the command line, 
// shunting off the various types of Options into
// different string containers. 
// Params contains the list of files to possibly expand (may contain wildcards.)

  procedure ReadCommandLine(RuntimeOptions, CompilerOptions, JikesOptions, Params : TStringList);
  var
    I, Count : Integer;
    Option, Param : String;
  begin
    Count := ParamCount;
    I := 0;
    Params.Sorted := True;
    Params.Duplicates := dupIgnore;
    while I < Count do
    begin
      inc(I);
      Param    := ParamStr(I);
      if (Param[1] = '/') or (Param[1]='-') then
      begin
        Option := Param;
        Option[1] := '-';
        if LowerCase(Option)='-nojikes' then
          nojikes := true
        else if (Option = '-v') or (Option = '-V') then
          isVerbose := True
        else if (Option = '-12') or (Option = '-1.2') or (Option = '-2') then
          TJavaRuntime.setJava11(False)
        else if (Option = '-11') or (Option = '-1.1') or (Option = '-1') then
          TJavaRuntime.setJava11(True)
        else if (Option = '-f') or (Option = '-F') then
          forceRecompile := True
        else if (Option = '-p') or (Option = '-P') then
          SetPauseOnexit(true)
        else if ((Option = '-d') or (Option = '-D')) and (I<Count) then
        begin
          outputDirectory := ParamStr(I+1);
          if outputDirectory[length(outputDirectory)] <> '\' then
              outputDirectory := outputDirectory + '\';
          CompilerOptions.add('-d');
          CompilerOptions.add(outputDirectory);
          inc(i);
          continue;
        end
        else if (LowerCase(Option)='-cp') or (LowerCase(Option) = '-classpath') then
        begin
          if I<Count then
            Option := Option + ' ' + ParamStr(I+1);
          TJavaRuntime.setAppClassPath(ParamStr(I+1));
          inc(I);
          continue;
        end
        else if (Copy(LowerCase(Option), 1, 2) = '-j') then
        begin
          if (Option[3] = '-') or (Option[3] = '/') then
          RuntimeOptions.add(Copy(Option, 3, length(Option)));
        end
        else begin // regular javac option to pass on
          Option := lowerCase(Option);
          if Option = '-o' then
              Option := '-O';
          CompilerOptions.add(Option); // now pass it on to javac.
        end;
      end
      else if Param[1] = '+' then
      begin
        JikesOptions.add(Param);
      end
      else
        Params.add(Param);
    end;
  end;

// Try to load the Jikes DLL and return a pointer to the main
// routine.

  procedure LoadJikes;
  begin
    JikesDLL := LoadLibrary('jikes.dll');
    if JikesDLL <> 0 then
        @JIkesMain := getProcAddress(JikesDLL, 'mainImpl')
  end;

  procedure RunJCImpl;
  var
    I : Integer;
    Params, CompilerOptions, JikesOptions, RuntimeOptions, Files : TStringList;
    Runtime : TJavaRuntime;
  begin
    Banner;
    CompilerOptions := TStringList.Create;
    RuntimeOptions := TStringList.Create;
    JikesOptions := TStringList.Create;
    Params := TStringList.Create;
    CompilerOptions.Duplicates := dupIgnore;
    JikesOptions.Duplicates := dupIgnore;
    Params.Duplicates := dupIgnore;
    RuntimeOptions.Duplicates := dupIgnore;
    ReadCommandLine(RuntimeOptions, CompilerOptions, JikesOptions, Params);
    if Params.Count=0 then
    begin //nothing to do. Exits.
      ShowUsage;
      ExitRoutine(1);
    end;

    if not NoJikes then
      LoadJikes;
    if @JikesMain = Nil then
      TJavaRuntime.setNeedTools(True);

    Runtime := TJavaRuntime.GetDefault;
    Files := TStringList.Create;
    Files.Sorted := true;
    Files.Duplicates := dupIgnore;

    I := 0;
    repeat
      addParam(Params[I], Files);
      inc(I);
    until I>= Params.Count;
    Params.Free;
    if Files.Count = 0 then
    begin
      if FoundAFile then
        writeln('No files are out of date. (Use -f option to force recompile.)');
      ExitRoutine(1);
    end
    else begin
      I := 0;
      while I<Files.Count do
      begin
          try
            Runtime.SanityCheckSource(Files[I]);
          except
            on EClasspathException do
            begin
              writeln('Warning: Ignoring source file ', Files[I]);
              writeln(Exception(ExceptObject).Message);
              writeln('');
              Files.Delete(I);
            end;
            end;
          inc(I);
      end;
    end;
    If Files.Count = 0 then
    begin
      writeln('No files to compile.');
      ExitRoutine(0);
      Exit;
    end;
    if isVerbose then
    begin
      for I := 1 to    Files.Count do
        writeln('Queueing: ', Files[I-1]);
      writeln('');
      writeln('Using Classpath: ', Runtime.Classpath);
      writeln('');
    end;

    writeln('Compiling ', Files.Count, ' file(s)');

    if nojikes then
      @JikesMain := Nil;

    if @JikesMain = Nil then
    begin
      Runtime.MinHeapSize := $1000000;
      Runtime.MaxHeapSize := $2000000;
      Runtime.ExitProc := @ExitRoutine;
      Runtime.processCommandLine(RuntimeOptions);
    end
    else begin
        CompilerOptions.add('-classpath');
        CompilerOptions.add(Runtime.ClassPath);
    end;
    CompilerOptions.addStrings(Files);
                                                                                                           
    if @JikesMain = Nil then
    begin
      Runtime.CallMain('sun.tools.javac.Main', CompilerOptions);
      Runtime.Wait;
    end
    else begin
      ShowJikesMessage;
      CompilerOptions.addStrings(JikesOptions);
      CompilerOptions.addStrings(Files);
      I := JikesMain(1 + CompilerOptions.Count, createArgv(CompilerOptions));
      ExitRoutine(I);
    end;
  end;
end.
