{
Copyright (c) 1998-2001 Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
     This product includes software developed by Jonathan Revusky
4. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.
5. The CopyrightNotice variable in the source code file SmartJGlobals.pas is not
   to be modified.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
}

unit install;

interface

    procedure _SetItUp; stdcall;

implementation

uses Windows, Registry, SysUtils, JavaRuntime;

var
  response : Integer;
  Reg : TRegistry;
  classKey, WindowsDir, Message : String;
  WindowsDirBuf : Array[0..255] of AnsiChar;
  Sun12, Sun11, MS : Boolean;

  function spaceToUnderLine(const S: String) : String; forward
  function setLaunchKey(reg : TRegistry; classKey, command, caption : String) : Boolean; forward;
  function newKey : TRegistry; forward;

  procedure findRuntimes;
  var
      RT : TJavaRuntime;
  begin
    try
      RT := TJavaRuntime.Create(SunJava1);
      Sun11 := true;
      RT.Free;
    except on Exception do
      Sun11 := false;
      end;
    try
      RT := TJavaRuntime.Create(SunJava2);
      Sun12 := true;
      RT.Free;
    except on Exception do
      Sun12 := false;
      end;
    try
      RT := TJavaRuntime.Create(MSJava);
      MS := true;
      RT.Free;
    except on Exception do
      MS := false;
      end;
  end;

  procedure CopyTheFile(filename : String);
  var
    Source, Dest : String;
  begin
    Source := ExtractFilePath(ParamStr(0)) + filename;
    Dest := WindowsDir + '\' + filename;
    if FileExists(Dest) then
      begin
          if MessageBox(0, Pchar(Dest + ' already exists. Overwrite?'), 'SmartJ setup', MB_YESNO) = IDNO then
              Exit;
        end;
    if not CopyFile(PChar(Source), Pchar(Dest), False) then
      MessageBox(0, PChar('Failure to copy to ' + Dest), 'SmartJ setup error', MB_OK);
  end;

  procedure _SetItUp; stdcall;
  var
    Command, DefaultCommand : String;
  begin
    getWindowsDirectory(WindowsDirBuf, 255);
    WindowsDir := String(WindowsDirBuf);
    Message := 'About to copy the SmartJ files to your Windows Directory (' + WindowsDir + ') Is this OK?';
    Response := MessageBox(0, Pchar(Message), 'SmartJ setup', MB_OKCANCEL);
    if Response = IDCANCEL then
    begin
      MessageBox(0, 'SmartJ setup aborted', 'SmartJ setup', MB_OK);
      Exit;
    end;

    CopyTheFile('smartj.exe');
    CopyTheFile('smartjw.exe');
    CopyTheFile('smartjc.exe');

    if FileExists(ExtractFilePath(ParamStr(0)) + 'smartjp.exe') then
      CopyTheFile('smartjp.exe');

    if FileExists(ExtractFilePath(ParamStr(0)) + 'jikes.dll') then
      CopyTheFile('jikes.dll');

    if FileExists(ExtractFilePath(ParamStr(0)) + 'jlaunch.dll') then
      CopyTheFile('jlaunch.dll');

    Response := MessageBox(0, 'Would you like to set up SmartJW to automatically launch java class files in the Windows Explorer?', 'SmartJ setup', MB_YESNO);
    if Response = IDNO then
        Exit;

    reg := newKey;
    if reg.OpenKey('.class', True) then
    begin
      classKey := reg.readString('');
      if classKey = '' then
        classKey := 'java_class_file';
      reg.WriteString('', classKey);
    end;
    reg.Free;
    reg := newKey;
    if reg.OpenKey(classKey, True) then
    begin
      if length(reg.readString('')) = 0 then
        reg.writeString('', 'java class');
    end;
    reg.Free;
    reg := newKey;
    FindRuntimes;
    if Sun12 then
    begin
      reg := newKey;
      DefaultCommand := 'launch with Java 2 VM';
      setLaunchKey(reg, classKey, WindowsDir + '\smartjw.exe ' + '"%1"', DefaultCommand);
      reg.Free;
    end;

    if Sun11 then
    begin
      reg := newKey;
      Command := 'launch with Sun Java 1.1 VM';
      setLaunchKey(reg, classKey, WindowsDir + '\smartjw.exe -1 ' + '"%1"', Command);
      if DefaultCommand = '' then
          DefaultCommand := Command;
      reg.Free;
    end;

    if MS then
    begin
        reg := newKey;
        Command := 'launch with Microsoft VM';
        setLaunchKey(reg, classKey, WindowsDir + '\smartjw.exe -m ' + '"%1"', Command);
        if DefaultCommand = '' then
          DefaultCommand := Command;
        reg.Free;
    end;

    reg := newKey;
    if reg.OpenKey(classKey + '\shell', true) then
    begin
      reg.writeString('', spaceToUnderLine(DefaultCommand));
    end;
    reg.Free;
    reg := newKey;
    if reg.OpenKey(classKey + '\shell\open\command', true) then
    begin
      reg.writeString('', WindowsDir + '\smartjw.exe "%1"');
    end;
    reg.Free;

    reg := newKey;
    if reg.OpenKey('Folder\shell\compile_java', true) then
        begin
            reg.writeString('', 'Recompile Java Source');
            if reg.openKey('\Folder\shell\compile_java\command', true) then
                reg.writeString('', 'smartjc.exe -p -v "%1"');
        end;
    reg.Free;
    MessageBox(0, 'Setup Complete. Enjoy. :-)', 'SmartJ Setup', MB_OK);
  end;

  function setLaunchKey(Reg : TRegistry; classkey, command, caption : String) : Boolean;
  begin
    result := false;
    spaceToUnderLine(caption);
    if reg.OpenKey(ClassKey + '\shell\' + spaceToUnderLine(caption), true) then
    begin
      reg.WriteString('', caption);
      if reg.OpenKey('command', true) then
      begin
        reg.WriteString('', command);
        result := true;
      end;
    end;
  end;

  function newKey : TRegistry;
  begin
    result := TRegistry.Create;
    result.RootKey := HKEY_CLASSES_ROOT;
  end;


  function spaceToUnderLine(const S: String) : String;
  var
      I : Integer;
  begin
    Result := S;
    for I := 1 to length(S) do
      if (Result[I] = ' ') then
          Result[I] := '_';
  end;
end.
