{
Copyright (c) 1998-2001 Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by Jonathan Revusky
4. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.
5. The CopyrightNotice variable in the source code file SmartJGlobals.pas is not
   to be modified.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
}
unit launch;

interface

  procedure RunImpl; 

implementation

uses Windows, Classes, Sysutils, ClassRead, SmartJGlobals,
    JavaRuntime, Jutils, JNIWrapper, ReadJar;

var
  isQuiet, Use11, UseMS, isZipped : Boolean;

const
  RevuskyClassLoader : String = 'com.revusky.smartj.SmartJClassLoader';

  // ParseArgs returns the first argument which is the Class file to
  // run. The Options container will contain the command-line
  // options and the Args container will contain the command-line arguments.
  
  function ParseArgs(Options, Args : TStrings) : String;
  var
    I : Integer;
    Param : String;
  begin
    I:=0;
    while I<ParamCount do 
    begin
      inc(I);
      Param := ParamStr(I);
      if (Result = '') and ((Param[1] =  '/') or (Param[1] = '-')) then 
      begin
        Param := LowerCase(Param);
        Param[1] := '-';
        if (Param = '-11') or (Param = '-1.1') or (Param = '-1') then 
          Use11 := true
        else if (Param = '-12') or (Param = '-1.2') or (Param = '-2') then 
        begin
          Use11 := false;
          UseMS := false;
        end
        else if (Param = '-m')  then 
        begin
          UseMS := true;
          Use11 := true;
        end
        else if (Param='-q') or (Param='-quiet') then 
          isQuiet := true
        else if (Param='-p') or (Param='-pause') then 
          setPauseOnexit(true)
        else if Param = '-classic' then
          TJavaRuntime.setClassicVM(true)
        else if Param = '-hotspot' then
          TJavaRuntime.setClassicVM(false)
        else if (Param = '-classpath') or (Param= '-cp') then  
        begin
          if I = ParamCount then 
            break;
          inc(I);
          Options.add(Param + ' ' + ParamStr(I));
          continue;
        end
        else 
          Options.add(ParamStr(I));
      end
      else 
      begin
        if Result ='' then 
          Result := Param
        else Args.add(Param);
      end;
    end;
  end;

  function getClassFromZip(Filename : TFilename) : TClassFile;
  var
    ZipFile : TZipFile;
    EntryName, ShortName, ArchiveName : String;
    EntryData : TStream;
    I : Integer;
  begin
    result := Nil;
    ZipFile := TZipFile.Create(Filename);
  // First look to see if it is specified in manifest.
    try
      EntryName := GetManifestMainClass(ZipFile);
      if EntryName <> '' then
      begin
        EntryName := dotToSlash(EntryName);
        AppendExtension(EntryName, '.class');
        EntryData := ZipFile.GetEntryDataByName(EntryName);
        if EntryData <> Nil then
        begin
          Result := TClassFile.Create(EntryData);
        end;
      end;
      if result <> Nil then
        for I :=  1 to ZipFile.Count do
        begin
          EntryName := ZipFile[I-1];
          if ExtractFileExt(EntryName) <> '.class' then
            Continue;
          EntryName := Copy(EntryName, 1, Length(EntryName) - 6);
          ShortName := ExtractFileName(toBackSlash(EntryName));
          ArchiveName := ExtractFileName(Filename);
          ArchiveName := Copy(ArchiveName, 1, length(ArchiveName) - 4);
          if LowerCase(ShortName) = lowerCase(ArchiveName) then
          begin
            EntryData := ZipFile.getEntryData(I-1);
            result := TClassFile.Create(EntryData);
            EntryData.Free;
            Break;
          end;
        end;
      finally
        ZipFile.Free;
    end;
  end;

  function GetClassFile(Filename : String) : TClassFile; stdcall;
  var
    Stream : TStream;
    Ext : String;
    ShortName : String;
  begin
    Ext := LowerCase(ExtractFileExt(Filename));
    if (Ext = '.zip') or (Ext = '.jar') then
    begin
      result := GetClassFromZip(Filename);
      IsZipped := result <> Nil;
      if IsZipped then
        TJavaRuntime.setBasepath(Filename)
      else
      begin
        ShortName := ExtractFileName(Filename);
        ShortName := Copy(ShortName, 1, length(ShortName) - length(Ext));
        Message(['Cannot find any class named '+ ShortName, 
        'in archive file: ' + Filename]);
        ExitRoutine(1);
      end;
    end
    else begin
      AppendExtension(Filename, '.class');
      Filename := ExpandFilename(Filename); 
      Stream := TFileStream.Create(Filename, 0);
      result := TClassFile.Create(Stream);
      Stream.Free;
    end;
  end;

  procedure ShowUsage;
  begin
    Message(UsageBanner);
  end;

  function RunApplet(classname : String) : boolean;
  var
    Runtime : TJavaRuntime;
    BaseAppletClass, FrameClass, ComponentClass, AppletClass : TJavaClass;
    FrameObject, AppletObject : TJavaObject;
    Frame_Show, Frame_Pack, Frame_Add, Applet_Init, Applet_Start, Frame_SetTitle: TJavaMethod;
    Params : TJavaParams;
    CloseHint : String;
  begin
// Get a handle to the Java runtime.
    Runtime := TJavaRuntime.GetDefault;
// Initialize some wrappers
    BaseAppletClass := TJavaClass.Create('java.applet.Applet');
    AppletClass := TJavaClass.Create(Classname);
    if AppletClass.extends(BaseAppletClass) then
    begin
      FrameClass := TJavaClass.Create('java.awt.Frame');
      ComponentClass := TJavaClass.Create('java.awt.Component');
      AppletObject := AppletClass.Instantiate(Nil);
      FrameObject := FrameClass.Instantiate(Nil);
      Frame_Show := TJavaMethod.CreateVoid(FrameClass, 'show');
      Frame_Pack:= TJavaMethod.CreateVoid(FrameClass, 'pack');
      Applet_Init := TJavaMethod.CreateVoid(AppletClass, 'init');
      Applet_Start := TJavaMethod.CreateVoid(AppletClass, 'start');
    
// Call the applet's init method.
      Applet_Init.Call(Nil, AppletObject);
    
// Add the applet to the frame.
      params := TJavaParams.Create;
      params.addString('Center');
      params.addObject(AppletObject, ComponentClass);
      Frame_Add := TJavaMethod.Create(FrameClass, 'add', nonstatic, AObject, params, ComponentClass);
      Frame_Add.Call(params, FrameObject);
      params.Free;
      params := TJavaParams.Create;
      params.addString('Applet: ' + ClassName);
      Frame_SetTitle := TJavaMethod.Create(FrameClass, 'setTitle', nonstatic, void, params, Nil);
      if isConsoleApp and not isQuiet then 
        VersionInfo(Classname);
    // Hello, Dolly.
      Frame_SetTitle.Call(params, FrameObject);
      Frame_Pack.Call(params, FrameObject);
      Frame_Show.Call(Nil, FrameObject);
// Call the start() method
      Applet_Start.Call(Nil, AppletObject);
    
    // Tell user he can abort with CTRL-C and sit back and wait.
      CloseHint := 'Use Task Manager to close window.';
      if isConsoleApp then
        CloseHint := 'Close the window via Ctrl-C in the console';
      message(['no main method specified', 'Running applet in Standalone Frame.', CloseHint]);
      Runtime.Wait;
      result := true;
    end
    else 
      result := false;
  end;
  
  procedure MainNotFound(classname : String);
  begin
    if not RunApplet(classname) then 
    begin
      Message(['Class ' + SlashToDot(ClassName) + ' is not an applet and has no', 
        'public static void main(String[] args) method defined.']);
      ExitRoutine(1);
    end;
  end;
  
  procedure ClassNotFound(classname : String; Filename : String);
  begin
    if Filename = '' then 
      Message(['Class ' + SlashToDot(ClassName) + ' not found.'])
    else begin
      Message(['Class ' + classname, 
        'in file ' + Filename + ' cannot be loaded.',
        'Probably a superclass or interface cannot be found.']);
    end;
  end;


  procedure RunSmarter(Filename : String; ArgsList, OptionsList : TStrings);
  var
    BasePath, PackageName, ClassName : String;
    FinalArgs : TStringList;
    Runtime : TJavaRuntime;
  begin
    TJavaRuntime.setMSJava(UseMS);
    TJavaRuntime.setJava11(Use11);
    Runtime := TJavaRuntime.GetDefault;
    PackageName := getPackageName(Filename);
    BasePath := Runtime.SanityCheckSource(Filename);
    Filename := ExtractFilename(Filename);
    ClassName := PackageName + FileName;
    ClassName := Copy(ClassName, 1, Length(ClassName)-5); // chop off .java extension
    FinalArgs := TStringList.Create;
    FinalArgs.Add(ClassName);
    if BasePath = '' then BasePath := '.';
    FinalArgs.Add(BasePath);
    FinalArgs.AddStrings(ArgsList);
    Runtime.ExitProc := @ExitRoutine;
    Runtime.ProcessCommandLine(OptionsList);
    Runtime.CallMain(RevuskyClassLoader, FinalArgs);
    Runtime.Wait;
  end;

  procedure RunImpl; 
  var
    ClassFile : TClassFile;
    OptionsList, Argslist : TStrings;
    Runtime : TJavaRuntime;
    Filename, ClassName, JavaHome, BasePath, Ext : String;
    IsArchive :Boolean;
  begin
    ArgsList := TStringList.Create;
    OptionsList := TStringList.Create;
    Filename := ParseArgs(OptionsList, ArgsList);
    if Filename = '' then 
    begin
      ShowUsage;
      Exit;
    end;
    if isConsoleApp and not isQuiet then 
      Message(CopyRightNotice);

    if LowerCase(ExtractFileExt(Filename)) = '.java' then
    begin
      try 
          RunSmarter(Filename, ArgsList, OptionsList);
          Exit;
      except
         on EJavaClassNotFound do 
         begin
           Filename := Copy(Filename, 1, Length(Filename) - 5);
         end;
       end;
    end;

    try
// try block will throw an exception if the classfile is not found.
      ClassFile := GetClassFile(Filename);
      if ClassFile<>Nil then
        ClassName := ClassFile.FullClassName;
    except
      on EFOpenError do
        begin
          ClassFile := Nil;
          ClassName := Filename;
          if isConsoleApp and not isQuiet then
          begin
            Ext := LowerCase(ExtractFileExt(Filename));
            IsArchive := (Ext ='.zip') or (Ext ='.jar');
            if not IsArchive then
              AppendExtension(Filename, '.class');
            writeln('Could not find file ' + Filename);
            if IsArchive then
              writeln('will attempt to launch class ' + filename)
            else
              writeln('will attempt to launch class ' + Copy(Filename, 1, Length(Filename)-6));
            writeln('');
          end;
          Filename := '';
        end;
     end;

    try
      TJavaRuntime.setMSJava(UseMS);
      TJavaRuntime.setJava11(Use11);
      Runtime := TJavaRuntime.GetDefault;
    except
      on F : EJavaRuntimeNotFound do
      begin
        Message(F.Message);
        ExitRoutine(1);
        Exit;
      end;

    end;
    Runtime.ExitProc := @ExitRoutine;
    Runtime.ProcessCommandLine(OptionsList);
    JavaHome := Runtime.JavaHome;

    if (not IsZipped) and (ClassFile <> Nil) then
    begin
      BasePath := Runtime.SanityCheck(Classfile.FullClassName, Filename);
      TJavaRuntime.setBasePath(BasePath);
    end;
    if Runtime.isMS then // seems necessary.
    begin
      setCurrentDirectory(PChar(BasePath));
    end;

    if ClassFile<>Nil then if not ClassFile.HasMain then 
    begin
      MainNotFound(ClassName);
    end;

    if isConsoleApp and not isQuiet then 
      VersionInfo(Classname);

    try 
      Runtime.CallMain(ClassName, ArgsList);
      Runtime.Wait;
    except 
      on EJavaClassNotFound do 
        ClassNotFound(Classname, Filename);
      on EJavaMethodNotFound do 
        MainNotFound(Classname);
      end;
    ArgsList.Free;
    ClassFile.Free;
  end;
  
end.



