{
Copyright (c) 1998 Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
     This product includes software developed by Jonathan Revusky
4. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.
5. The CopyrightNotice variable in the code is not to be modified.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
}

unit SmartJGlobals;

interface

  procedure _Run; stdcall;
  procedure _RunJC; stdcall;
  procedure _SetConsoleApp(b : boolean); stdcall;
  function isConsoleApp : Boolean;
  procedure SetPauseOnExit(b : Boolean);
  function isPauseOnExit : Boolean;
  procedure Message(const arr : array of String);
  procedure VersionInfo(Classname : String);
  procedure AbortRoutine; stdcall;
  procedure ExitRoutine(exitCode : Integer); stdcall;
  //function getPackageName(filename : String) : String;
  function isOutOfDate(filename, outputdirectory : String) : Boolean;
  procedure AppendExtension(var Filename  : String ; Extension : String);
  procedure AppendBackSlash(var S: String);
   

{
The following Copyright notice and the first 2 lines of the Usage Banner are not to be
modified without permission.
}

const
   CopyrightNotice  : array[1..3] of String = (
               'Smart Java launcher version 0.60 (C) Jonathan Revusky 1998,99',
               'SmartJ Open Source project: http://www.revusky.com/hacks',
               '');

   UsageBanner : array [1..24] of String = (
               'Smart Java launcher version 0.60 (C) Jonathan Revusky 1998,99',
               'SmartJ Open Source project: http://www.revusky.com/hacks', 
              'Usage: SmartJ [-options] Classfile [arguments]',
              ' Options:',
                  '  -v, -verbose        turn on verbose mode',
                  '  -verbosegc          print a message when garbage collection occurs',
                  '  -noasync            disable asynchronous garbage collection',
                  '  -noclassgc          disable garbage collection of classes',
                  '  -ss<number>         set the maximum stack size for any thread',
                  '  -oss<number>        set the maximum Java stack for any thread',
                  '  -ms<number>         set the initial Java heap size',
                  '  -mx<number>         set the maximum Java heap size',
                  '  -D<name>=<value>    set a system property',
                  '  -classpath <path>   add <path> to the base classpath',
                  '  -verify             verify all classes when loaded',
                  '  -verifyremote       verify classes loaded from the network',
                  '  -noverify           do not verify any classes',
                  '  -nojit              disable JIT compiler',
                  '  -q, -quiet          disable SmartJ status info',
                  '  -1                  use Java 1.1 if available',
                  '  -2                  use Java 1.2 if available',
                  '  -classic            do not use Hotspot if it is available',
                  '  -m                  use Microsoft VM if available',
                  '  -p                  pause before exiting'
                  );

implementation

uses Windows, SysUtils, JavaRuntime, Compile, Launch, Jutils;

var
  ConsoleApp, PauseOnExit : Boolean;


  procedure _Run; stdcall;
  begin
    try
      RunImpl;
    except
      on EC: EclasspathException do
      begin
        Message(EC.Message);
        ExitRoutine(1);
        Exit;
      end;
      on Exception do
        message(['Unexpected Error Occurred', Exception(ExceptObject).Message]);
    end;
    ExitRoutine(0);
  end;
    
  procedure _RunJC; stdcall;
  begin
    try
      RunJCImpl;
    except
      on Exception do
        writeln(Exception(ExceptObject).Message);
  end;
  end;
   
  procedure _SetConsoleApp(b : Boolean); stdcall;
  begin
    ConsoleApp := b;
  end;
   
  function isConsoleApp : Boolean;
  begin
    result := ConsoleApp;
  end;
   
  procedure SetPauseOnExit(b : Boolean);
  begin
    PauseOnExit := b;
  end;

  function IsPauseOnExit : Boolean;
  begin
    result := PauseOnExit;
  end;
   
  procedure Message(const arr : array of String);
  var
    I : Integer;
    msg : String;
  begin
    if isConsoleApp then 
    begin
      for I:= Low(arr) to High(arr) do
        writeln(arr[I]);
    end
    else 
    begin
      msg := '';
      for I:=Low(arr) to High(arr) do
        msg := msg + #13 + #10 + arr[I];
      MessageBox(0, PChar(msg), 'SmartJ Java Launcher', mb_OK);
    end;
  end;
   
  procedure AbortRoutine; stdcall;
  var
    S: String;
  begin
    if isConsoleApp then writeln('Process aborted.');
    if PauseOnExit and isConsoleApp then
    begin
      writeln('Press Enter to Continue...');
      readln(S);
    end;
    Halt;
  end;
   
  procedure ExitRoutine(exitCode : Integer); stdcall;
  var
    S: String;
  begin
    if PauseOnExit and isConsoleApp then
    begin
      writeln('Press Enter to Continue...');
      readln(S);
    end;
    Halt(exitCode);
  end;

  procedure VersionInfo(Classname : String);
  var
    Version, Vendor : String;
    JRT : TJavaRuntime;
  begin
    JRT := TJavaRuntime.getDefault;
    Version := '1.2 Classic VM';
    if JRT.Hotspot then Version := '1.2 Hotspot VM';
    Vendor := 'Sun';
    if JRT.isJava11 or JRT.isMS then 
      Version := '1.1 VM';
    if JRT.isMS then 
      Vendor := 'Microsoft';
    Message(['launching class ' + SlashToDot(ClassName),
        'using the ' + Vendor + ' java ' + Version + ' in: ' + JRT.JavaHome,
        'with classpath: ' + JRT.Classpath,
        '']);
  end;
   
   //puts a backslash on the end of the string if there is not one.
   
  procedure appendBackslash(var S: String);
  begin
    if S[length(S)] <> '\' then
      S := S + '\';
  end;
   
  function isOutOfDate(filename, outputdirectory : String) : Boolean;
  var
     ClassFile, package : String;
  begin
    ClassFile := Copy(filename, 1, length(filename) -4) + 'class';
    if length(outputDirectory) >0 then 
    begin
      appendBackslash(outputDirectory);
      package := getPackageName(filename);
      ClassFile := outputdirectory + toBackSlash(package) + ExtractFileName(ClassFile);
    end;
    Result := FileAge(filename)>FileAge(ClassFile)
  end;

  procedure AppendExtension(var Filename: String ; Extension : String);
  begin
    if ExtractFileExt(Filename) <> Extension then
    begin
      if Filename[Length(Filename)] = '.' then
          Filename := Copy(Filename, 1, Length(Filename) -1);
      Filename := Filename + Extension;
    end;
  end;

end.
